/*
 *  include/ub/io_acct.h
 *
 *  Copyright (C) 2006 SWsoft
 *  All rights reserved.
 *  
 *  Licensing governed by "linux/COPYING.SWsoft" file.
 *
 *  Pavel Emelianov <xemul@openvz.org>
 *
 */

#ifndef __UB_IO_ACCT_H_
#define __UB_IO_ACCT_H_

#ifdef CONFIG_UBC_IO_ACCT
#include <ub/beancounter.h>
#include <ub/ub_page.h>

#define page_iopb(page)	({			\
		struct page_beancounter *pb;	\
		pb = page_pbc(page);		\
		rmb();				\
		pb;				\
	})

/*
 * IO ub is required in task context only, so if exec_ub is set
 * to NULL this means that uses doesn't need to charge some
 * resources. nevertheless IO activity must be accounted, so we
 * account it to current's task beancounter.
 */

#define get_io_ub()	({				\
		struct user_beancounter *___ub;		\
		___ub = get_exec_ub();			\
		if (unlikely(___ub == NULL))		\
			___ub = get_task_ub(current);	\
		___ub;					\
	})

extern struct page_beancounter **page_pblist(struct page *);

extern void ub_io_save_context(struct page *, size_t);
extern void __ub_io_release_context(struct page *,
		struct page_beancounter *, size_t);

static inline void ub_io_release_context(struct page *pg,
		struct page_beancounter *pb, size_t size)
{
	if (pb != NULL)
		__ub_io_release_context(pg, pb, size);
}

static inline void ub_io_account_read(size_t bytes)
{
	ub_percpu_add(get_io_ub(), bytes_read, bytes);
}

static inline void ub_io_account_write(size_t bytes)
{
	ub_percpu_add(get_io_ub(), bytes_wrote, bytes);
}

static inline void ub_io_account_dirty(struct page *page, size_t bytes)
{
	ub_io_save_context(page, bytes);
}

static inline void ub_io_account_write_cancelled(size_t bytes)
{
	ub_percpu_add(get_io_ub(), bytes_cancelled, bytes);
}

void ub_init_io(kmem_cache_t *);
#else /* UBC_IO_ACCT */
#define page_iopb(page)		(NULL)
#define page_pblist(page)	(&page_pbc(page))

static inline void ub_io_release_context(struct page *pg,
		struct page_beancounter *pb, size_t bytes)
{
}

static inline void ub_io_account_dirty(struct page *p, size_t bytes)
{
}

static inline void ub_io_account_read(size_t bytes)
{
}

static inline void ub_io_account_write(size_t bytes)
{
}

static inline void ub_io_account_write_cancelled(size_t bytes)
{
}

static inline void ub_init_io(kmem_cache_t *pb_cachep) { };
#endif

#ifdef CONFIG_UBC_DEBUG_IO
extern void ub_io_release_debug(struct page *pg);
#else
#define ub_io_release_debug(pg)	do { } while (0)
#endif
#endif
